@@ -27,41 +27,39 @@
 
 from __future__ import annotations
 
-from .rpid import verify_rp_id
+import logging
+import os
+from typing import Any, Callable, Mapping, Sequence
+
+from cryptography.exceptions import InvalidSignature as _InvalidSignature
+from cryptography.hazmat.primitives import constant_time
+
 from .cose import CoseKey
-from .utils import websafe_encode, websafe_decode
+from .rpid import verify_rp_id
+from .utils import websafe_decode, websafe_encode
 from .webauthn import (
-    CollectedClientData,
-    AuthenticatorData,
+    AttestationConveyancePreference,
     AttestationObject,
     AttestedCredentialData,
-    AttestationConveyancePreference,
-    PublicKeyCredentialRpEntity,
-    PublicKeyCredentialUserEntity,
-    AuthenticatorSelectionCriteria,
-    PublicKeyCredentialDescriptor,
-    PublicKeyCredentialType,
-    PublicKeyCredentialParameters,
-    PublicKeyCredentialCreationOptions,
-    PublicKeyCredentialRequestOptions,
-    UserVerificationRequirement,
-    ResidentKeyRequirement,
-    AuthenticatorAttachment,
-    RegistrationResponse,
     AuthenticationResponse,
+    AuthenticatorAttachment,
+    AuthenticatorData,
+    AuthenticatorSelectionCriteria,
+    CollectedClientData,
     CredentialCreationOptions,
     CredentialRequestOptions,
+    PublicKeyCredentialCreationOptions,
+    PublicKeyCredentialDescriptor,
+    PublicKeyCredentialParameters,
+    PublicKeyCredentialRequestOptions,
+    PublicKeyCredentialRpEntity,
+    PublicKeyCredentialType,
+    PublicKeyCredentialUserEntity,
+    RegistrationResponse,
+    ResidentKeyRequirement,
+    UserVerificationRequirement,
 )
 
-from cryptography.hazmat.primitives import constant_time
-from cryptography.exceptions import InvalidSignature as _InvalidSignature
-from dataclasses import replace
-from urllib.parse import urlparse
-from typing import Sequence, Mapping, Optional, Callable, Union, Tuple, Any, overload
-
-import os
-import logging
-
 logger = logging.getLogger(__name__)
 
 
@@ -73,7 +71,7 @@ def _verify_origin_for_rp(rp_id: str) -> VerifyOrigin:
     return lambda o: verify_rp_id(rp_id, o)
 
 
-def _validata_challenge(challenge: Optional[bytes]) -> bytes:
+def _validata_challenge(challenge: bytes | None) -> bytes:
     if challenge is None:
         challenge = os.urandom(32)
     else:
@@ -97,15 +95,15 @@ def to_descriptor(
     :rtype: PublicKeyCredentialDescriptor
     """
     return PublicKeyCredentialDescriptor(
-        PublicKeyCredentialType.PUBLIC_KEY, credential.credential_id, transports
+        type=PublicKeyCredentialType.PUBLIC_KEY,
+        id=credential.credential_id,
+        transports=transports,
     )
 
 
 def _wrap_credentials(
-    creds: Optional[
-        Sequence[Union[AttestedCredentialData, PublicKeyCredentialDescriptor]]
-    ],
-) -> Optional[Sequence[PublicKeyCredentialDescriptor]]:
+    creds: Sequence[AttestedCredentialData | PublicKeyCredentialDescriptor] | None,
+) -> Sequence[PublicKeyCredentialDescriptor] | None:
     if creds is None:
         return None
     return [
@@ -139,16 +137,19 @@ class Fido2Server:
     def __init__(
         self,
         rp: PublicKeyCredentialRpEntity,
-        attestation: Optional[AttestationConveyancePreference] = None,
-        verify_origin: Optional[VerifyOrigin] = None,
-        verify_attestation: Optional[VerifyAttestation] = None,
+        attestation: AttestationConveyancePreference | None = None,
+        verify_origin: VerifyOrigin | None = None,
+        verify_attestation: VerifyAttestation | None = None,
     ):
         self.rp = PublicKeyCredentialRpEntity.from_dict(rp)
+        assert self.rp.id is not None  # nosec
         self._verify = verify_origin or _verify_origin_for_rp(self.rp.id)
         self.timeout = None
         self.attestation = AttestationConveyancePreference(attestation)
         self.allowed_algorithms = [
-            PublicKeyCredentialParameters(PublicKeyCredentialType.PUBLIC_KEY, alg)
+            PublicKeyCredentialParameters(
+                type=PublicKeyCredentialType.PUBLIC_KEY, alg=alg
+            )
             for alg in CoseKey.supported_algorithms()
         ]
         self._verify_attestation = verify_attestation or _ignore_attestation
@@ -157,15 +158,15 @@ class Fido2Server:
     def register_begin(
         self,
         user: PublicKeyCredentialUserEntity,
-        credentials: Optional[
-            Sequence[Union[AttestedCredentialData, PublicKeyCredentialDescriptor]]
-        ] = None,
-        resident_key_requirement: Optional[ResidentKeyRequirement] = None,
-        user_verification: Optional[UserVerificationRequirement] = None,
-        authenticator_attachment: Optional[AuthenticatorAttachment] = None,
-        challenge: Optional[bytes] = None,
+        credentials: (
+            Sequence[AttestedCredentialData | PublicKeyCredentialDescriptor] | None
+        ) = None,
+        resident_key_requirement: ResidentKeyRequirement | None = None,
+        user_verification: UserVerificationRequirement | None = None,
+        authenticator_attachment: AuthenticatorAttachment | None = None,
+        challenge: bytes | None = None,
         extensions=None,
-    ) -> Tuple[CredentialCreationOptions, Any]:
+    ) -> tuple[CredentialCreationOptions, Any]:
         """Return a PublicKeyCredentialCreationOptions registration object and
         the internal state dictionary that needs to be passed as is to the
         corresponding `register_complete` call.
@@ -193,18 +194,18 @@ class Fido2Server:
 
         return (
             CredentialCreationOptions(
-                PublicKeyCredentialCreationOptions(
-                    self.rp,
-                    PublicKeyCredentialUserEntity.from_dict(user),
-                    challenge,
-                    self.allowed_algorithms,
-                    self.timeout,
-                    descriptors,
-                    (
+                public_key=PublicKeyCredentialCreationOptions(
+                    rp=self.rp,
+                    user=PublicKeyCredentialUserEntity.from_dict(user),
+                    challenge=challenge,
+                    pub_key_cred_params=self.allowed_algorithms,
+                    timeout=self.timeout,
+                    exclude_credentials=descriptors,
+                    authenticator_selection=(
                         AuthenticatorSelectionCriteria(
-                            authenticator_attachment,
-                            resident_key_requirement,
-                            user_verification,
+                            authenticator_attachment=authenticator_attachment,
+                            resident_key=resident_key_requirement,
+                            user_verification=user_verification,
                         )
                         if any(
                             (
@@ -215,57 +216,29 @@ class Fido2Server:
                         )
                         else None
                     ),
-                    self.attestation,
-                    extensions,
+                    attestation=self.attestation,
+                    extensions=extensions,
                 )
             ),
             state,
         )
 
-    @overload
     def register_complete(
         self,
         state,
-        response: Union[RegistrationResponse, Mapping[str, Any]],
+        response: RegistrationResponse | Mapping[str, Any],
     ) -> AuthenticatorData:
-        pass
-
-    @overload
-    def register_complete(
-        self,
-        state,
-        client_data: CollectedClientData,
-        attestation_object: AttestationObject,
-    ) -> AuthenticatorData:
-        pass
-
-    def register_complete(self, state, *args, **kwargs):
         """Verify the correctness of the registration data received from
         the client.
 
         :param state: The state data returned by the corresponding
             `register_begin`.
-        :param client_data: The client data.
-        :param attestation_object: The attestation object.
+        :param response: The registration response from the client.
         :return: The authenticator data
         """
-        response = None
-        if len(args) == 1 and not kwargs:
-            response = args[0]
-        elif set(kwargs) == {"response"} and not args:
-            response = kwargs["response"]
-        if response:
-            registration = RegistrationResponse.from_dict(response)
-            client_data = registration.response.client_data
-            attestation_object = registration.response.attestation_object
-        else:
-            names = ["client_data", "attestation_object"]
-            pos = dict(zip(names, args))
-            data = {**kwargs, **pos}
-            if set(kwargs) & set(pos) or set(data) != set(names):
-                raise TypeError("incorrect arguments passed to register_complete()")
-            client_data = data[names[0]]
-            attestation_object = data[names[1]]
+        registration = RegistrationResponse.from_dict(response)
+        client_data = registration.response.client_data
+        attestation_object = registration.response.attestation_object
 
         if client_data.type != CollectedClientData.TYPE.CREATE:
             raise ValueError("Incorrect type in CollectedClientData.")
@@ -276,7 +249,7 @@ class Fido2Server:
         ):
             raise ValueError("Wrong challenge in response.")
         if not constant_time.bytes_eq(
-            self.rp.id_hash, attestation_object.auth_data.rp_id_hash
+            self.rp.id_hash or b"", attestation_object.auth_data.rp_id_hash
         ):
             raise ValueError("Wrong RP ID hash in response.")
         if not attestation_object.auth_data.is_user_present():
@@ -306,13 +279,13 @@ class Fido2Server:
 
     def authenticate_begin(
         self,
-        credentials: Optional[
-            Sequence[Union[AttestedCredentialData, PublicKeyCredentialDescriptor]]
-        ] = None,
-        user_verification: Optional[UserVerificationRequirement] = None,
-        challenge: Optional[bytes] = None,
+        credentials: (
+            Sequence[AttestedCredentialData | PublicKeyCredentialDescriptor] | None
+        ) = None,
+        user_verification: UserVerificationRequirement | None = None,
+        challenge: bytes | None = None,
         extensions=None,
-    ) -> Tuple[CredentialRequestOptions, Any]:
+    ) -> tuple[CredentialRequestOptions, Any]:
         """Return a PublicKeyCredentialRequestOptions assertion object and the internal
         state dictionary that needs to be passed as is to the corresponding
         `authenticate_complete` call.
@@ -336,40 +309,24 @@ class Fido2Server:
 
         return (
             CredentialRequestOptions(
-                PublicKeyCredentialRequestOptions(
-                    challenge,
-                    self.timeout,
-                    self.rp.id,
-                    descriptors,
-                    user_verification,
-                    extensions,
+                public_key=PublicKeyCredentialRequestOptions(
+                    challenge=challenge,
+                    timeout=self.timeout,
+                    rp_id=self.rp.id,
+                    allow_credentials=descriptors,
+                    user_verification=user_verification,
+                    extensions=extensions,
                 )
             ),
             state,
         )
 
-    @overload
     def authenticate_complete(
         self,
         state,
         credentials: Sequence[AttestedCredentialData],
-        response: Union[AuthenticationResponse, Mapping[str, Any]],
+        response: AuthenticationResponse | Mapping[str, Any],
     ) -> AttestedCredentialData:
-        pass
-
-    @overload
-    def authenticate_complete(
-        self,
-        state,
-        credentials: Sequence[AttestedCredentialData],
-        credential_id: bytes,
-        client_data: CollectedClientData,
-        auth_data: AuthenticatorData,
-        signature: bytes,
-    ) -> AttestedCredentialData:
-        pass
-
-    def authenticate_complete(self, state, credentials, *args, **kwargs):
         """Verify the correctness of the assertion data received from
         the client.
 
@@ -381,27 +338,11 @@ class Fido2Server:
         :param auth_data: The authenticator data.
         :param signature: The signature provided by the client."""
 
-        response = None
-        if len(args) == 1 and not kwargs:
-            response = args[0]
-        elif set(kwargs) == {"response"} and not args:
-            response = kwargs["response"]
-        if response:
-            authentication = AuthenticationResponse.from_dict(response)
-            credential_id = authentication.id
-            client_data = authentication.response.client_data
-            auth_data = authentication.response.authenticator_data
-            signature = authentication.response.signature
-        else:
-            names = ["credential_id", "client_data", "auth_data", "signature"]
-            pos = dict(zip(names, args))
-            data = {**kwargs, **pos}
-            if set(kwargs) & set(pos) or set(data) != set(names):
-                raise TypeError("incorrect arguments passed to authenticate_complete()")
-            credential_id = data[names[0]]
-            client_data = data[names[1]]
-            auth_data = data[names[2]]
-            signature = data[names[3]]
+        authentication = AuthenticationResponse.from_dict(response)
+        credential_id = authentication.raw_id
+        client_data = authentication.response.client_data
+        auth_data = authentication.response.authenticator_data
+        signature = authentication.response.signature
 
         if client_data.type != CollectedClientData.TYPE.GET:
             raise ValueError("Incorrect type in CollectedClientData.")
@@ -409,7 +350,7 @@ class Fido2Server:
             raise ValueError("Invalid origin in CollectedClientData.")
         if websafe_decode(state["challenge"]) != client_data.challenge:
             raise ValueError("Wrong challenge in response.")
-        if not constant_time.bytes_eq(self.rp.id_hash, auth_data.rp_id_hash):
+        if not constant_time.bytes_eq(self.rp.id_hash or b"", auth_data.rp_id_hash):
             raise ValueError("Wrong RP ID hash in response.")
         if not auth_data.is_user_present():
             raise ValueError("User Present flag not set.")
@@ -434,87 +375,9 @@ class Fido2Server:
 
     @staticmethod
     def _make_internal_state(
-        challenge: bytes, user_verification: Optional[UserVerificationRequirement]
+        challenge: bytes, user_verification: UserVerificationRequirement | None
     ):
         return {
             "challenge": websafe_encode(challenge),
             "user_verification": user_verification,
         }
-
-
-# TODO 2.0: Delete, or move to an example
-def verify_app_id(app_id: str, origin: str) -> bool:
-    """Checks if a FIDO U2F App ID is usable for a given origin.
-
-    :param app_id: The App ID to validate.
-    :param origin: The origin of the request.
-    :return: True if the App ID is usable by the origin, False if not.
-
-    .. deprecated:: 1.2.0
-       This will be removed in python-fido2 2.0.
-    """
-    url = urlparse(app_id)
-    hostname = url.hostname
-    # Note that FIDO U2F requires a secure context, i.e. an origin with https scheme.
-    # However, most browsers also treat http://localhost as a secure context. See
-    # https://groups.google.com/a/chromium.org/g/blink-dev/c/RC9dSw-O3fE/m/E3_0XaT0BAAJ
-    if url.scheme != "https" and (url.scheme, hostname) != ("http", "localhost"):
-        return False
-    if not hostname:
-        return False
-    return verify_rp_id(hostname, origin)
-
-
-# TODO 2.0: Delete, or move to an example
-class U2FFido2Server(Fido2Server):
-    """Fido2Server which can be used with existing U2F credentials.
-
-    This Fido2Server can be used with existing U2F credentials by using the
-    WebAuthn appid extension, as well as with new WebAuthn credentials.
-    See https://www.w3.org/TR/webauthn/#sctn-appid-extension for details.
-
-    :param app_id: The appId which was used for U2F registration.
-    :param verify_u2f_origin: (optional) Alternative function to validate an
-        origin for U2F credentials.
-
-    For other parameters, see Fido2Server.
-
-    .. deprecated:: 1.2.0
-       This will be removed in python-fido2 2.0.
-    """
-
-    def __init__(
-        self,
-        app_id: str,
-        rp: PublicKeyCredentialRpEntity,
-        verify_u2f_origin: Optional[VerifyOrigin] = None,
-        *args,
-        **kwargs,
-    ):
-        super().__init__(rp, *args, **kwargs)
-        if verify_u2f_origin:
-            kwargs["verify_origin"] = verify_u2f_origin
-        else:
-            kwargs["verify_origin"] = lambda o: verify_app_id(app_id, o)
-        self._app_id = app_id
-        self._app_id_server = Fido2Server(
-            replace(PublicKeyCredentialRpEntity.from_dict(rp), id=app_id),
-            *args,
-            **kwargs,
-        )
-
-    def register_begin(self, *args, **kwargs):
-        kwargs.setdefault("extensions", {})["appidExclude"] = self._app_id
-        req, state = super().register_begin(*args, **kwargs)
-        return req, state
-
-    def authenticate_begin(self, *args, **kwargs):
-        kwargs.setdefault("extensions", {})["appid"] = self._app_id
-        req, state = super().authenticate_begin(*args, **kwargs)
-        return req, state
-
-    def authenticate_complete(self, *args, **kwargs):
-        try:
-            return super().authenticate_complete(*args, **kwargs)
-        except ValueError:
-            return self._app_id_server.authenticate_complete(*args, **kwargs)
